/*******************************************************/
/* HW-5400 with AD9850 DDS (100 mhz osc)               */
/* Freescale (HC9S12DJ64)  0X3C9                       */
/*                                                     */
/* BFO IF board                                        */
/* Written: Darrin Kohn (KE9NS)                        */
/* Sofware Ver: 1.0.1005    10/25/05                   */
/*                                                     */
/* rebuilt Ver 2.0     2/10/2012                       */
/*         from IF DDS board hiwave raw code           */
/*******************************************************/

#include <hidef.h>
#include <register.h>
#include <stdlib.h>
#include <ctype.h>
#include <String.h>
#include <stdio.h>
#include <math.h>

#pragma CODE_SEG DEFAULT

// --------- PORTATD0            J2 4pos molex plug ( 1=+5v, 2=AD01, 3=AD00, 4=GND )
// PAD7 = 
// PAD6 = 
// PAD5 = 
// PAD4 =
// PAD3 =
// PAD2 =
// PAD1 =                        IF-Shift KNOB from front panel of radio
// PAD0 =   


// --------- PORTA               J6 9pos molex plug ( 1=GND, 2-9 = PA7-PA0 )
// PA7 = O                        0= 4k X select      1-0-1
// PA6 = O                        0= 2k Y-X select    1-0-1
// PA5 = O                        0= 6k Y select      1-0-1                 
// PA4 =  
// PA3 =  
// PA2 =  
// PA1 =  
// PA0 =  


// --------- PORTB               J4 9pos molex plug ( 1=GND, 2-9 = PB0-PB7 )
// PB7 =  
// PB6 =  
// PB5 =  
// PB4 =  
// PB3 =  
// PB2 =  
// PB1 =  
// PB0 = O                        1= IF KNOB changed value



// --------- PORTE               Not connected
// PE7 =
// PE6 = 
// PE5 = 
// PE4 =
// PE3 =
// PE2 =
// PE1 = 
// PE0 = 


// --------- PORTM               Not connected
// PM5 = 
// PM4 =
// PM3 =
// PM2 =
// PM1 = 
// PM0 = 

                                    
// --------- PORTP               Not connected
// PP7 = 
// PP5 =  
// PP4 = 
// PP3 = 
// PP2 = 
// PP1 =  
// PP0 = 
				 

// --------- PORTT              Connected to IF DDS chip AD9850 & 100mhz tcox 
// PT7 = 
// PT6 = 
// PT5 = 
// PT4 = 
// PT3 = 
// PT2 =  O                     DDS freq update (pulse 0-1-0)
// PT1 =  O                     DDS serial clk  (pulse 0-1-0)
// PT0 =  O                     DDS serial data (40 bit= 32data & 8bit control)


// --------- PORTJ              J3 2pos molex connector (1-2 = PJ7-PJ6 )
// PJ7 =  I                     
// PJ6 =  I                     MODE SELECT 0=AM,1=CW,2=LSB,3=USB 


// --------- PORTS              U10 4pos molex connector ( 1-4 = PS3-PS0 )
// PTS3 =  I                     tx signal
// PTS2 =  I                     
// PTS1 =  I                     From LO DDS PTP4-7: 0=WIDE 4k, 1=NARROW  
// PTS0 =  I                     From LO DDS PTP0-3: 0=EESB 6k, 1=NARROW 



/*=======================================================================================================*/

void DDS_UPDATE(void);                                      // send data to AD9850 chip

/*******************************************************/
/*     .C function declarations                        */
/*******************************************************/

#define ulng unsigned long             
#define uint unsigned int
#define uchr unsigned char                
                                         

/*=======================================================================================================*/

const ulng AM_IF =(ulng)8830000;                            // 4048 AM  IF freq 8.830.000                  0x0086bc30  0x4048


// 2.8k filter  (2.825 khz wide @ 6db down)
const ulng USB_IF =(ulng)8831400;    // USB IF freq 8.831.400  (8831400 -6db)  0x0086c1a8
const ulng LSB_IF =(ulng)8828600;    // LSB IF freq 8.828.400  (8828600 -6db)  0x0086b6b8
const ulng CW_IF =(ulng)8830700;     // CW IF freq 8.830.700                   0x0086beec

// 4.0k filter  (4.275 khz wide @ 6db down)
const ulng USB4_IF =(ulng)8832138;   // USBW IF freq 8.832.000 (8832138 -6db)  0x0086c400
const ulng LSB4_IF =(ulng)8827863;   // LSBW IF freq 8.828.000 (8827863 -6db)  0x0086b460
const ulng CW4_IF =(ulng)8830700;    // CWW IF freq 8.830.700                  0x0086beec  0x4060

// 6.0k filter  (6.450 khz wide @ 6db down)
const ulng USB6_IF =(ulng)8833225;   // USBW IF freq 8.833.000 (8833225 -6db) 0x0086c400
const ulng LSB6_IF =(ulng)8826775;   // LSBW IF freq 8.827.000 (8826775 -6db) 0x0086b460
const ulng CW6_IF =(ulng)8830700;    // CWW IF freq 8.830.700                 0x0086beec  



// 2.8k filter  (2.825 khz wide @ 6db down)
const ulng USB_IFR =(ulng)8831400;    // USB IF freq 8.831.400  (8831400 -6db)  0x0086c1a8
const ulng LSB_IFR =(ulng)8828600;    // LSB IF freq 8.828.400  (8828600 -6db)  0x0086b6b8

// 4.0k filter  (4.275 khz wide @ 6db down)
const ulng USB4_IFR =(ulng)8832000;   // USBW IF freq 8.832.000 (8832138 -6db)  0x0086c400
const ulng LSB4_IFR =(ulng)8828000;   // LSBW IF freq 8.828.000 (8827863 -6db)  0x0086b460

// 6.0k filter  (6.450 khz wide @ 6db down)
const ulng USB6_IFR =(ulng)8833000;   // USBW IF freq 8.833.000 (8833225 -6db) 0x0086c400
const ulng LSB6_IFR =(ulng)8827000;   // LSBW IF freq 8.827.000 (8826775 -6db) 0x0086b460


const double FREQ_CLOCK =(double)99999550;                  // 99999850 4064 ref clock for AD9850  0x4cbebc0d double  0x4064
const double FREQ_BITS  =0xffffffff;                        // 4068 largest phase value   0x4f800000 double  0x4068
 
 

int  CENTERF_TX = 505;                                         // 0x800,1        IF value from 0 to 500 to 1024
int  CENTERF_RX=505;


uint  CHANGE = 1;                                           // 0x802,3        1= IF Knob changed

uint  RLYONCNT4 = 2000;                                      // 0x804,5        RLYOFFCNT starts at 2000
uint  RLYONCNT2 = 2000;
uint  RLYONCNT6 = 2000;


volatile uint  AD0=0;                                       // 0x806,7
volatile uint  AD1=0;                                       // 0x806,7
                                                             
ulng  DDS=0;                                                // 0x808,9,a,b    INT freq sent to update routine
ulng  LASTDDS=0;                                            // 0x80c,d,e,f    last DDS value (to prevent repeating update)

					
uchr TEMP=0;                                                // 0x810          mode 0=AM, 1=CW, 2=USB, 3=LSB 

ulng x=0;                                                   // 0x811-0x814    delay RLYOFFCNT
ulng y=0;                                                   // 0x815-0x818    delay RLYOFFCNT


uint AD0_LAST=0;                                            // 0x819,a        last A/D value used
uint AD00=0;                                                // 0x81b,c        A/D value

uint AD3=0;                                                 // 0x81d,e        A/D holders to check for movement of IF-Shift knob
uint AD4=0;                                                 // 0x81f,820


uint AD0_LASTR=0;                                            // 0x819,a        last A/D value used
uint AD00R=0;                                                // 0x81b,c        A/D value


uint AD3R=0;                                                 // 0x81d,e        A/D holders to check for movement of IF-Shift knob
uint AD4R=0;                                                 // 0x81f,820


uchr WIDE=2;                                                // 0x821     0=2.4khz, 1=4.0khz filter  (force narrow at startup ?)

//uint RLYOFFCNT4=0;                                           // 0x822,3   RLYOFFCNT starts at 2000
//uint RLYOFFCNT2=0;  
//uint RLYOFFCNT6=0;  

uchr IFOFF = 0;         // 1=NO SHIFT, 0=SHIFT

/*=======================================================================================================*/



#pragma CODE_SEG DEFAULT

/*======================================================================================================*/
/* Main routine                                                                                         */
/*======================================================================================================*/

void main(void)

{ 
       
        _IRQCR = 0;
      
        _REFDV = 7;                                         // Divides PLLCLK by (REFDV + 1) (7=24mhz) (7)
        _SYNR = 46;                                         // Multiplies PLLCLK by 2(SYNR + 1) (5=192) (46)

        for (;;)
        {
          if ( (_CRGFLG & _SCM)==0 )
          {
            if ( (_CRGFLG & _LOCK)==_LOCK ) break;
          }
        }

        _CLKSEL = _PLLSEL;                                  // System clock is now PLLCLK (0X80=pll on)

        _ATDCTL2 = _ADPU;                                   // AD turned on
        _ATDCTL3 = _S4C + _S1C;                             // 5 conversions per sequence
        _ATDCTL4 = _SMP1+_SMP0+_PRS2+_PRS0;                 // AD bus speed = 2mhz 10bit
        _ATDCTL5 = _MULT;                                   // 0x85 right justified,multi channel
        
            
            
        _PORTA = 0xf0;                                      // 4094  PORTA (0x0000)  set (PA7-6-5 all high)
        _DDRA = 0xff;                                       // all output                 ( Used to signal MODE and Filter Width ? )
 
    
    
        _PORTB = 0x00;                                      // PORTB (0x0001)                  (NOT USED on IF BOARD)
        _DDRB = 0xff;                                       // bit 7-4=ouptut, bit3-0 input

 
 
        _PTT = 0x00;								                    		// PORTT (0x240)
        _DDRT = 0xff;                                       // all output                      (communication with AD9850 chip)
       
        _PTJ =0;                                            // PORTJ  (0x268)
        _PERJ = 0xff;
        _DDRJ = 0;                                          // all input                       ( Mode selector input )
      
        _PTS = 0;                                           // PORTS (0x248)                   ( signals from LO DDS board: )
        _DDRS = 0x00;                                       // input                           (which filter and IF shift ON/OFF)
        _PERS = 0xff;
      
      
      
      
        _PEAR = 0x10;                                       // porte assignment                (NOT USED on IF BOARD)
       
        _INITEE = 0x11;                                     // eeprom starts at 1000h          (NOT USED on IF BOARD)
        _ECLKDIV =0x16;                                     // EEPROM clk (22)
       
   
     //   _RTICTL = 0x10;                                     // RTI every 244uSec             (NOT USED on IF BOARD)
     //   _CRGFLG = _RTIF + _LOCKIF + _SCMIF;                 // clear flags
     //   _CRGINT = _RTIE + _LOCKIE + _SCMIE;                 // enable real time interrupt, and self clock mode
           
           
/*================================================================================*/
       

       
        DDS = AM_IF;                                     // 40cc
       
        DDS_UPDATE();                                    // 40cf
       
        for (x=1; x< 0xff;x++)                           // 40d2-4120    Startup Delay ?
        {
            for (y=1;y < 0xff;y++)
            {
                
            asm nop;
            asm nop;
            }
            
        }
    
        
        DDS_UPDATE();                                    // 4122
        DDS_UPDATE();                                    // 4125
            
        
        for (;;)                                         //  4128
        {
           
                TEMP = ((_PTJ & 192) >> 6);              // 4128        mode 0=AM, 1=CW, 2=USB, 3=LSB   
                
                
 
//----------------------------------------------------------------------------------               
// Signal FILTER RELAY CIRCUIT
//----------------------------------------------------------------------------------               
               
       
              if ((_PTS & 3)==1)                          // 4k
              {
                   if (WIDE == 2)
                   {
                       RLYONCNT4 = 2000;
                   }
                   WIDE = 1;
                  
              }
              else if ((_PTS & 3)==3)                   // 2.4k
              {
                   if (WIDE == 1)
                   {
                      RLYONCNT2 = 2000;
                   }
                   WIDE = 0;
              
              }
              else if ((_PTS & 3)==2)                      // 6k
              {
                   if (WIDE == 0)
                   {
                      RLYONCNT6 = 2000;
                   }
                   WIDE = 2;
              }
       
       
                 
        
               if (WIDE==1)
               {
                  if (RLYONCNT4)
                  {
                      RLYONCNT4--;                              //               
                      _PORTA = ((_PORTA & 0x0F) | 0x70);
                  }
                  else
                  {
                      _PORTA = ((_PORTA & 0x0F) | 0xF0);    // after either 2000 cycle turn PA7-5 = 1 (remove voltage from relays)
                  }
                  
               }
               else if (WIDE==0)
               {
                  if (RLYONCNT2)
                  {
                      RLYONCNT2--;                              //               
                     _PORTA = ((_PORTA & 0x0F) | 0xB0);
                  }
                  else
                  {
                      _PORTA = ((_PORTA & 0x0F) | 0xF0);    // after either 2000 cycle turn port back to 1 (remove voltage from relays)
                  }
               }
               else if (WIDE==2)
               {
                  
                    if (RLYONCNT6)
                    {
                        RLYONCNT6--;                              //               
                        _PORTA = ((_PORTA & 0x0F) | 0xD0);
                    }
                    else
                    {
                        _PORTA = ((_PORTA & 0x0F) | 0xF0);    // after either 2000 cycle turn port back to 1 (remove voltage from relays)
                    }
               }
                
//----------------------------------------------------------------------------------               
// Select Filter and MODE Values
//----------------------------------------------------------------------------------               
                
                 
          	    if ((TEMP==0))                            // 4186     check MODE of operation   mode 0=AM, 1=CW, 2=USB, 3=LSB 
          	    {
          	        DDS = AM_IF;                          // 418b     This mode not used
          	        _PORTA = ((_PORTA & 0xF0) | 0x01);    // 4197     output to show mode
          	    }
          	    else if ((TEMP==1))                       // 419f     CW MODE
          	    {
          	       
           	        if (WIDE==1)                             // 41a3
          	        {
          	             DDS = CW4_IF;                   // 41a8     Select WIDE if WIDE>0
          	        }
          	        else if(WIDE==0)
          	        {
          	             DDS = CW_IF;                    // 41b3
          	        }
          	        else if (WIDE==2)
          	        {
          	             DDS = CW6_IF;                     // 
          	        }
          	        
          	          _PORTA = ((_PORTA & 0xF0) | 0x02);  // 41bf     output to show mode
          	       
          	    }
          	    else if ((TEMP==2))                       // 41c7      USB MODE
          	    {
          	      
          	        if (WIDE==1)                             // 41cb
          	        {
          	           if (_PTS & 8)  DDS = USB4_IF;                    // 41d0
          	           else DDS = USB4_IFR; 
          	           
          	        }
          	        else if(WIDE==0)
          	        {
          	      	   if (_PTS & 8)  DDS = USB_IF;                    // 41d0
          	           else DDS = USB_IFR; 
                 
          	        }
          	        else if (WIDE == 2)
          	        {
           	           if (_PTS & 8)  DDS = USB6_IF;                    // 41d0
          	           else DDS = USB6_IFR; 
         	        }
          	     
          	        
          	         _PORTA = ((_PORTA & 0xF0) | 0x04);   // 4202      output to show mode
           	    }
          	    else                                                  // LSB MODE
          	    {
          	      
          	        if (WIDE==1)                             // 41e6
          	        {
          	           if (_PTS & 8)  DDS = LSB4_IF;                    // 41d0
          	           else DDS = LSB4_IFR; 
         	        }
          	        else if (WIDE==0)
          	        {
           	           if (_PTS & 8)  DDS = LSB_IF;                    // 41d0
          	           else DDS = LSB_IFR; 
         	        }
          	        else if (WIDE == 2)
          	        {
       	               if (_PTS & 8)  DDS = LSB6_IF;                    // 41d0
          	           else DDS = LSB6_IFR; 
             	        }
          	     
   
          	         _PORTA = ((_PORTA & 0xF0) | 0x04);   // 4202      output to show mode
           	    }
           	    
           	    
//----------------------------------------------------------------------------------               
// IF-Shift Knob reading
//----------------------------------------------------------------------------------               
          	    
           	    
           	  
           	    if (_ATDSTAT0 & _SCF)                      // 420a
                { 
                   AD0= (_ATDDR0)>>6;                      // 420e     Read A/D value
                   AD1= (_ATDDR1)>>6;  
                  
                   _ATDCTL5 = _MULT;                       // 4219     restart A/D
                }                                          //          AD ready to read
           
        			
        				 AD3=AD0;                                  // 421d     AD3 = 81D        *2: center=1000 *1: center=500
        				 AD3R=AD1; 
                             
              
                 if ((AD3 > AD0_LAST+6) || (AD3 < AD0_LAST-6))
                 {
                    AD0_LAST=AD3;                           // 4234
                		AD4=AD3;                                // 423a
                	  CHANGE=1;                               //         A/D value changed
                 }
          	   
          	 
          	 
                if ((AD3R > AD0_LASTR+6) || (AD3R < AD0_LASTR-6))
                 {
                    AD0_LASTR=AD3R;                           // 4234
                		AD4R=AD3R;                                // 423a
                	  CHANGE=1;                               //         A/D value changed
                 }
          	 
          	 
          	   
//----------------------------------------------------------------------------------               
// IF-Shift Knob adjusting
//----------------------------------------------------------------------------------               
           	   
           	   
           if (IFOFF==0)
           {
            
           	     if ((_PTS & 8))                            //  PTS3=1: TX IF shift
           	     {
          	        if (AD4 <=CENTERF_TX)                      // 424b
                    {
                        AD00 = CENTERF_TX - AD4;               // 4259          value = 500 - (0 to 1024)
                        DDS = DDS - AD00;                  // 4265
                     } 
                    else 
                    {
        	              AD00 = AD4-CENTERF_TX;                // 4271
                        DDS = DDS + AD00;                  //4283
                    }
                    
          	     }  // TX
          	     else
          	     {
          	      
          	        if (AD4R <=CENTERF_RX)                      // 424b
                    {
                        AD00R = CENTERF_RX - AD4R;               // 4259          value = 500 - (0 to 1024)
                        DDS = DDS - AD00R;                  // 4265
                     } 
                    else 
                    {
        	              AD00R = AD4R-CENTERF_RX;                // 4271
                        DDS = DDS + AD00R;                  //4283
                    }
          
         	      
          	     }  // RX
           	    
           }
//----------------------------------------------------------------------------------               
// Update DDS chip
//----------------------------------------------------------------------------------               
          	  
          	    
          	     if (DDS != LASTDDS)                      // 428e (lower word)
          	     {
          	        LASTDDS=DDS;                          // 429f
                    DDS_UPDATE();                         // 42a2
          	     }
          	     
       
//----------------------------------------------------------------------------------               
// Show if IF Changed
//----------------------------------------------------------------------------------               
      
                 if (CHANGE)                              // 42a5   change=802
                 {
                    _PORTB = 1;                          // 42ac
                    CHANGE++;                            // 42b2
                 } 
                 else 
                 {                                       // 42b8
                    _PORTB = 0;
                 }
            
        } // for loop forever                           // jmp 4128
        
} // main
 
 




 
/*======================================================================================================*/
/* Send DDS freq to AD9850 chip                                                                         */
/* Convert Freq. to DDS (with 100 mhz ref)                                                              */
// PT2 =  4   DDS freq update (pulse 0-1-0)
// PT1 =  2   DDS serial clk  (pulse 0-1-0)
// PT0 =  1   DDS serial data (40 bit= 32data & 8bit control)
/*======================================================================================================*/
void DDS_UPDATE(void)
{
   	
      static double FREQ_F;				                         // 824,5,6,7										
      static ulng FREQ_I;			                             // 828,9,a,b
      static ulng TEMP;                                    // 82c,d,e,f
           
          
      FREQ_I = DDS;					                               // 42c1
                      
      FREQ_F = ((double)FREQ_I * FREQ_BITS) / FREQ_CLOCK;  // 42ef          serial phase = (freq * 2^32) / clock
      
	    FREQ_I = FREQ_F;				                             // 42f8
  			
      for (TEMP=1;TEMP!=0;TEMP=TEMP<<1)			               // 4302
      {
           if ((FREQ_I & TEMP)) _PTT = 1;                  // 4329
           else _PTT = 0;                                  // 432e
  				 
          _PTT = _PTT | 2;                                 // 4331
          _PTT = _PTT & ~2;                                // 4335           DDS serial CLK
      }
  
       
      for (TEMP=1;TEMP <=8;TEMP++)                          //               this code loops unrolled
      {
          _PTT=0;                                           // 434f          control and phase = 0
      
          _PTT = 2;                                         // 4354
          _PTT = 0;                                         // 4357
      }
    
      
      _PTT = 4;                                             // 43a5
       asm nop;
      _PTT =0;                                              // 43a9           DDS freq update
      
    
}                                                           // 43ac


 
 
#pragma CODE_SEG INTERRUPT_ROUTINES
#pragma TRAP_PROC
void Trap1(void);                                               // trap all unknown interrupts
                                      
/*--------------------------------------------*/
/* Trap all unintentional interrupts          */
/*--------------------------------------------*/                                      
interrupt void Trap1(void)
{
     
        asm SEI;                                                // turn off interrupts
      
        _PORTK = 0;                                             // shut off valve relays
        _PTT = 0;                                               // shut off valve relays
        _PTH = 0;                                               // turn off all PLS outputs
      
        _ARMCOP = 0x55;
        _ARMCOP = 0xaa;
      
                 
        for(;;)
        {
      
        _ARMCOP = 0x55;
        _ARMCOP = 0xaa;
        //do nothing
        }
}
   
 